//+------------------------------------------------------------------+
//|                                                       multi lsma |
//+------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers  2
#property indicator_color1   DeepSkyBlue
#property indicator_color2   PaleVioletRed
#property indicator_width1   2
#property indicator_width2   2
#property indicator_minimum -5
#property indicator_maximum  5

//
//
//
//
//

extern string TimeFrame        = "Current time frame";
extern int    Period1          =  8;
extern int    Period2          = 11;
extern int    Period3          = 14;
extern int    Period4          = 17;
extern int    Period5          = 20;
extern int    Price            = PRICE_CLOSE;
extern bool   alertsOn         = false;
extern bool   alertsOnCurrent  = false;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;
extern bool   ShowArrows       = false;
extern string arrowsIdentifier = "multilsma arrows";
extern color  arrowsUpColor    = DeepSkyBlue;
extern color  arrowsDnColor    = Red;

//
//
//
//
//

double bufferUp[];
double bufferDn[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,bufferUp); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,bufferDn); SetIndexStyle(1,DRAW_HISTOGRAM);
   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //
 
   IndicatorShortName(timeFrameToString(timeFrame)+" multi lsma ("+Period1+","+Period2+","+Period3+","+Period4+","+Period5+")");
}
int deinit()
{
   if (!calculateValue && ShowArrows) deleteArrows();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][10];
int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { bufferUp[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double trendUp = 0;
         double trendDn = 0;
            checkTrend(i,r,Period1,0,5,trendUp,trendDn);
            checkTrend(i,r,Period2,1,6,trendUp,trendDn);
            checkTrend(i,r,Period3,2,7,trendUp,trendDn);
            checkTrend(i,r,Period4,3,8,trendUp,trendDn);
            checkTrend(i,r,Period5,4,9,trendUp,trendDn);
         bufferUp[i] = trendUp;
         bufferDn[i] = trendDn;
         manageArrow(i);
      }
      manageAlerts();
      return(0);
   }   
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         bufferUp[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Period1,Period2,Period3,Period4,Period5,Price,0,y);
         bufferDn[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Period1,Period2,Period3,Period4,Period5,Price,1,y);
         manageArrow(i);
   }
   manageAlerts();
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void checkTrend(int i, int r, int period, int fromValues, int forTrend, double& sumUp, double& sumDn)
{
   work[r][fromValues] = 3.0*iMA(NULL,0,period,0,MODE_LWMA,Price,i)-2.0*iMA(NULL,0,period,0,MODE_SMA,Price,i);
   work[r][forTrend]   = work[r-1][forTrend];
      if (work[r][fromValues]>work[r-1][fromValues]) work[r][forTrend]= 1;
      if (work[r][fromValues]<work[r-1][fromValues]) work[r][forTrend]=-1;
      if (work[r][forTrend]>0) sumUp++;
      if (work[r][forTrend]<0) sumDn--;
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (bufferUp[whichBar] ==  5 && bufferUp[whichBar+1]!= 5) doAlert(whichBar,"up");
      if (bufferDn[whichBar] == -5 && bufferDn[whichBar+1]!=-5) doAlert(whichBar,"down");
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," multi lsma trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"multi lsma"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (!calculateValue && ShowArrows)
   {
      deleteArrow(Time[i]);
      if (bufferUp[i] ==  5 && bufferUp[i+1]!= 5) drawArrow(i,arrowsUpColor,241,false);
      if (bufferDn[i] == -5 && bufferDn[i+1]!=-5) drawArrow(i,arrowsDnColor,242,true);
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}
void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}